import os
import random
from mastermind.exceptions import InvalidUserInput
from mastermind.utils import load_heading

MODULE_PATH = os.path.dirname(os.path.abspath(__file__))


class MasterMind:

    def __init__(self):
        self.player_name = "player"
        self.count_correct = 0
        self.count_incorrect = 0
        self.end_game = False
        self.heading = load_heading(os.path.join(MODULE_PATH, "heading.txt"))

    def run(self):
        self.print_intro()
        while not self._game_should_end():
            self.next_game_round()
        self._print_stats(self.count_correct, self.count_incorrect)

    def next_game_round(self):
        number, guess = self._new_round()
        while self._round_continue(guess, number):
            user_input = input("Your guess?: ")
            if self._user_ends_game(user_input):
                self.end_game = True
                break
            try:
                guess = self._parse_guess(user_input)
            except InvalidUserInput:
                print("Please choose number from 1 to 10")
                break
            if number == guess:
                print("CORRECT!")
                self.count_correct += 1
                break
            else:
                print("INCORRECT!")
                self.count_incorrect += 1

    def set_player_name(self, name):
        self.player_name = name

    def print_intro(self):
        print(self.heading)
        self.select_player_name()
        print()
        print("All right {}".format(self.player_name))
        print("The rules are following:")
        print(" * I pick a number")
        print(" * You'll guess which number I'm thinking of")
        print(" * If you want to end the game, just write 'end' or 'quit'")
        print()
        print("At the end of the game, you'll see the stats of your gameplay")
        print()

    def select_player_name(self):
        player_name = input("What's your name?: ")
        self.set_player_name(player_name)

    def _game_should_end(self):
        return self.end_game

    def _user_ends_game(self, user_input):
        user_input = user_input.lower().strip()
        if user_input == "end" or user_input == "quit":
            return True
        return False

    def _new_round(self):
        print()
        print("I think of a number from 1 to 10")
        number = random.randint(1, 10)
        guess = -1
        return number, guess

    def _round_continue(self, guess, number):
        return guess != number and not self.end_game

    def _parse_guess(self, user_input):
        try:
            guess = int(user_input)
        except ValueError as e:
            raise InvalidUserInput(e)
        if guess <= 0 or guess > 10:
            raise InvalidUserInput("Guess is out of range 1 - 10")
        return guess

    def _print_stats(self, correct, incorrect):
        total = correct + incorrect
        print()
        print("+{:-^20}+".format("your score"))
        print("| {: <10} | {: 5d} |".format("correct", correct))
        print("| {: <10} | {: 5d} |".format("incorrect", incorrect))
        print("| {: <10} | {: 5d} |".format("total", total))
        print("+{:=^20}+".format(""))
        print()
